"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var format_1 = require("./format");
var unitValueArithmetic_1 = require("./unitValueArithmetic");
var Measure = /** @class */ (function () {
    // Instance methods
    function Measure(value, unit, symbol) {
        this.value = value;
        this.unit = unit;
        this.symbol = symbol;
    }
    // Construction functions
    Measure.dimension = function (dim, symbol) {
        return new Measure(1, unitValueArithmetic_1.dimension(dim, symbol), symbol);
    };
    Measure.dimensionless = function (value) {
        return new Measure(value, {});
    };
    Measure.of = function (value, quantity, symbol) {
        return new Measure(value * quantity.value, quantity.unit, symbol);
    };
    Measure.unsafeConstruct = function (value, unit, symbol) {
        return new Measure(value, unit, symbol);
    };
    Measure.prototype.withSymbol = function (symbol) {
        return new Measure(this.value, this.unit, symbol);
    };
    Measure.prototype.getSymbol = function () {
        return this.symbol;
    };
    Measure.prototype.getUnit = function () {
        return this.unit;
    };
    // Arithmetic
    Measure.prototype.normalized = function () {
        return new Measure(1, this.unit);
    };
    Measure.prototype.plus = function (other) {
        return new Measure(this.value + other.value, this.unit);
    };
    Measure.prototype.minus = function (other) {
        return new Measure(this.value - other.value, this.unit);
    };
    Measure.prototype.negate = function () {
        return new Measure(-this.value, this.unit);
    };
    Measure.prototype.scale = function (value) {
        var numericValue = typeof value === "number" ? value : value.value;
        return new Measure(numericValue * this.value, this.unit);
    };
    Measure.prototype.times = function (other) {
        return new Measure(this.value * other.value, unitValueArithmetic_1.multiplyUnits(this.unit, other.unit));
    };
    Measure.prototype.over = function (other) {
        return new Measure(this.value / other.value, unitValueArithmetic_1.divideUnits(this.unit, other.unit));
    };
    Measure.prototype.per = function (other) {
        return this.over(other);
    };
    Measure.prototype.inverse = function () {
        return new Measure(1 / this.value, unitValueArithmetic_1.exponentiateUnit(this.unit, -1));
    };
    Measure.prototype.reciprocal = function () {
        return this.inverse();
    };
    // Comparisons
    Measure.prototype.compareTo = function (other) {
        return this.value - other.value;
    };
    Measure.prototype.isLessThan = function (other) {
        return this.compareTo(other) < 0;
    };
    Measure.prototype.isLessThanOrEqualTo = function (other) {
        return this.compareTo(other) <= 0;
    };
    Measure.prototype.isEqualTo = function (other) {
        return this.compareTo(other) === 0;
    };
    Measure.prototype.isNotEqualTo = function (other) {
        return this.compareTo(other) !== 0;
    };
    Measure.prototype.isGreaterThanOrEqualTo = function (other) {
        return this.compareTo(other) >= 0;
    };
    Measure.prototype.isGreaterThan = function (other) {
        return this.compareTo(other) > 0;
    };
    // Formatting
    Measure.prototype.toString = function () {
        return "" + this.value + format_1.formatUnit(this.unit);
    };
    Measure.prototype.in = function (unit) {
        if (unit.symbol === undefined) {
            return this.toString();
        }
        var value = this.value / unit.value;
        return value + " " + unit.symbol;
    };
    return Measure;
}());
exports.Measure = Measure;
Measure.prototype.squared = function () {
    return pow(this, 2);
};
Measure.prototype.cubed = function () {
    return pow(this, 3);
};
function square(measure) {
    return pow(measure, 2);
}
exports.square = square;
function cubic(measure) {
    return pow(measure, 3);
}
exports.cubic = cubic;
function pow(measure, power) {
    return Measure.unsafeConstruct(Math.pow(measure.value, power), unitValueArithmetic_1.exponentiateUnit(measure.getUnit(), power));
}
exports.pow = pow;
//# sourceMappingURL=measure.js.map