import { AddendOf, AddExponents, ArithmeticError, DivideExponents, Exponent, IsArithmeticError, MultiplicandOf, MultiplyExponents, ProductOf } from "../exponent";
export declare type UnitWithSymbols<U extends Unit = Unit> = {
    [D in keyof U]+?: [string, NonNullable<U[D]>];
};
export declare type SymbolAndExponent = [string, Exponent];
export declare type Unit = Partial<{
    [dimension: string]: Exponent;
}>;
/** Returns the product of two units. This is the sum of two dimension vectors. */
export declare type MultiplyUnits<L extends Unit, R extends Unit> = HandleErrors<{
    [Dim in keyof L | keyof R]: AddExponents<GetExponent<L, Dim>, GetExponent<R, Dim>>;
}>;
/** A type that is assignable from all units that can be multiplied by U without producing an error. */
export declare type MultiplicandUnit<U extends Unit> = Partial<{
    [D in keyof U]: AddendOf<NonNullable<U[D]>>;
}> & Unit;
/** Returns the quotient of two units. This is the difference of two dimension vectors. */
export declare type DivideUnits<L extends Unit, R extends Unit> = HandleErrors<{
    [Dim in keyof L | keyof R]: AddExponents<GetExponent<L, Dim>, MultiplyExponents<GetExponent<R, Dim>, -1>>;
}>;
/** A type that is assignable from all units that U can be divided by without producing an error. */
export declare type DivisorUnit<U extends Unit> = MultiplicandUnit<ExponentiateUnit<U, -1>>;
/** Returns the unit raised to a power. This is the scalar multiple of the dimension vector. */
export declare type ExponentiateUnit<U extends Unit, N extends Exponent> = HandleErrors<{
    [Dim in keyof U]: MultiplyExponents<GetExponent<U, Dim>, N>;
}>;
/** A type that is assignable from all units that can be raised to the N without producing an error. */
export declare type BaseUnit<N extends Exponent> = Partial<{
    [dimension: string]: MultiplicandOf<N>;
}>;
/** Returns the nth root of a unit. This is the inverse scalar multiple of the dimension vector. */
export declare type NthRootUnit<U extends Unit, N extends Exponent> = HandleErrors<{
    [Dim in keyof U]: DivideExponents<GetExponent<U, Dim>, N>;
}>;
/** A type that is assignable from all units whose Nth root does not produce an error. */
export declare type RadicandUnit<D extends Exponent> = Partial<{
    [dimension: string]: ProductOf<D>;
}>;
/** Handle errors in the result of an arithmetic operation. */
declare type HandleErrors<Result extends ArithmeticResult> = true extends ResultHasError<Result> ? ArithmeticError : StripZeroes<{
    [Dim in keyof Result]: Result[Dim] extends ArithmeticError ? 0 : Result[Dim];
}>;
declare type ResultHasError<Result> = {
    [Dim in keyof Result]: IsArithmeticError<Result[Dim]>;
}[keyof Result];
declare type ArithmeticResult = {
    [dimension: string]: Exponent | ArithmeticError;
};
/** Removes all zero exponent dimensions from a dimension vector */
declare type StripZeroes<U extends Unit> = Pick<U, NonZeroKeys<U>>;
declare type NonZeroKeys<U extends Unit> = {
    [Dim in keyof U]: NonNullable<U[Dim]> extends 0 ? never : Dim;
}[keyof U];
declare type GetExponent<U extends Unit, K> = K extends keyof Unit ? (undefined extends U[K] ? 0 : NonNullable<U[K]>) : 0;
export {};
